#include "Chronos.h"

Chronos::Chronos() {
	Timer1.initialize(100);
	Timer1.attachInterrupt(timerHandler);
}

void Chronos::timerHandler(void) {
	me().processTimerHandler();
}

int8_t Chronos::attachMsCallback(void (*callbackFunction)(), uint16_t milliseconds) {
	return me().attachCallback(callbackFunction, milliseconds, false);
}

int8_t Chronos::attachUsCallback(void (*callbackFunction)(), uint16_t microseconds) {
	int8_t id = me().attachCallback(callbackFunction, microseconds, true);

	if (id >= 0) {
		me().usCallbackCount++;
	}

	return id;
}

int8_t Chronos::attachMsMemberCallback(ChronosCallbackOwnerClass* obj, uint16_t milliseconds) {
	return me().attachMemberCallback(obj, milliseconds, false);
}

int8_t Chronos::attachUsMemberCallback(ChronosCallbackOwnerClass* obj, uint16_t microseconds) {
	int8_t id = me().attachMemberCallback(obj, microseconds, true);

	if (id >= 0) {
		me().usCallbackCount++;
	}

	return id;
}

void Chronos::detachCallback(uint8_t id) {
	if (id >= 0) {
		me().deleteCallbackItem(id);
	}
}
 
void Chronos::delay(uint16_t delay_ms) {
	me().delayLimit = delay_ms;

	while(me().delayCounter < me().delayLimit) {
		__asm__ __volatile__ (" nop\n");
	}

	me().delayCounter = 0;
	me().delayLimit = 0;
}

/***************************************************************/
/************************   Private   **************************/
/***************************************************************/

void Chronos::processTimerHandler(void) {
	if (this->lock == true) {
		return;
	}
	
	if (this->usCallbackCount > 0) {
		this->walkCallback(true);
	}

	this->usCounter++;

	if (this->usCounter == 10) {

		this->usCounter = 0;

		this->walkCallback(false);

		if (this->delayLimit != 0
			&& this->delayCounter < this->delayLimit
		) {
			this->delayCounter++;
		}
	}
}

int8_t Chronos::attachLambda(void (*callbackFunction)(), uint16_t period, bool isUsScale) {
	if (this->callbackArraySize == 127)
		return -1;

	ChronosCallbackItem item;
	item.obj = nullptr;
	item.method = &callbackFunction;
	item.id = this->spawnNewId();
	item.counter = 0;
	item.limit = period;
	item.isUsScale = isUsScale;

	return this->pushCallbackItem(&item);
}

int8_t Chronos::attachCallback(void (*callbackFunction)(), uint16_t period, bool isUsScale) {
	if (this->callbackArraySize == 127)
		return -1;

	ChronosCallbackItem item;
	item.obj = nullptr;
	item.method = &callbackFunction;
	item.id = this->spawnNewId();
	item.counter = 0;
	item.limit = period;
	item.isUsScale = isUsScale;

	return this->pushCallbackItem(&item);
}

int8_t Chronos::attachMemberCallback(ChronosCallbackOwnerClass* obj, uint16_t period, bool isUsScale) {
	if (this->callbackArraySize == 127)
		return -1;

	ChronosCallbackItem item;
	item.obj = obj;
	item.method = nullptr;
	item.id = this->spawnNewId();
	item.counter = 0;
	item.limit = period;
	item.isUsScale = isUsScale;

	return this->pushCallbackItem(&item);
}

int8_t Chronos::pushCallbackItem(ChronosCallbackItem *item) {
	this->lock = true;

	ChronosCallbackItem *tmpCallbackArray = (ChronosCallbackItem*) realloc(this->callbackArray, (sizeof * callbackArray) * (this->callbackArraySize + 1));
	
	if (tmpCallbackArray) {
		this->callbackArray = tmpCallbackArray;
		this->callbackArray[this->callbackArraySize] = *item;

		this->callbackArraySize++;
		this->lock = false;

		return this->callbackArraySize - 1;
	}

	this->lock = false;

	return -1;
}

void Chronos::walkCallback(bool isUsScale) {
	for(uint8_t i = 0; i < this->callbackArraySize; i++) {
		if (this->callbackArray[i].isUsScale == isUsScale) {
			this->callbackArray[i].counter += isUsScale ? TICK_SIZE : 1;

			if (this->callbackArray[i].limit != 0
				&& this->callbackArray[i].counter >= this->callbackArray[i].limit
			) {
				this->callbackArray[i].counter = 0;

				if (this->callbackArray[i].obj != nullptr) {
					this->callbackArray[i].obj->chronosHandler();
				} else if (this->callbackArray[i].method != nullptr) {
					(*this->callbackArray[i].method)();
				}
			}
		}
	}
}

int8_t Chronos::deleteCallbackItem(uint8_t id) {
	bool found = false;
	uint8_t i = 0;

	for(i = 0; i < this->callbackArraySize; i++) {
		if (this->callbackArray[i].id == id) {
			found = true;
			break;
		}
	}

	if (found) {
		this->lock = true;

		ChronosCallbackItem *tmp = (ChronosCallbackItem*) malloc((sizeof * callbackArray) * (this->callbackArraySize - 1));

		if (tmp) {
			uint8_t j = 0;

			for(i = 0; i < this->callbackArraySize; i++) {
				if (this->callbackArray[i].id != id) {
					tmp[j] = this->callbackArray[i];
					j++;
				}
			}

			free(this->callbackArray);
			this->callbackArray = tmp;

			this->callbackArraySize--;
		}

		this->lock = false;
	}
	
	return this->callbackArraySize;
}

uint8_t Chronos::spawnNewId() {
	bool found = false;

	for (uint8_t j = 0; j <= 127; j++) {
		found = false;

		for(uint8_t i = 0; i < this->callbackArraySize; i++) {
			if (this->callbackArray[i].id == j) {
				found = true;
			}
		}

		if (!found)
			return j; //127 max
	}

	return -1;
}
