#ifndef _GEEGROW_CHRONOS_H
#define _GEEGROW_CHRONOS_H
#include <Arduino.h>
#include <TimerOne.h>
#define TICK_SIZE 100

typedef void (*UserCallback)(void);

class ChronosCallbackOwnerClass {
	public:
		virtual void chronosHandler() = 0;
};

struct ChronosCallbackItem
{ 
    ChronosCallbackOwnerClass* obj;
    UserCallback* method;
    uint16_t counter;
    uint16_t limit;
    uint8_t id;
    bool isUsScale;
};


class Chronos {
	public:

		static Chronos& me() {
	        static Chronos  instance;
	        return instance;
	    }

	    template < typename NonCaptureLambda >
	    static int8_t attachMsLambda(NonCaptureLambda lambda, uint16_t milliseconds) {
	    	return me().attachLambda(lambda, milliseconds, false);
	    }

	    template < typename NonCaptureLambda >
	    static int8_t attachUsLambda(NonCaptureLambda lambda, uint16_t microseconds) {
	    	return me().attachLambda(lambda, microseconds, true);
	    }

		static int8_t attachMsCallback(void (*callbackFunction)(), uint16_t milliseconds);
		static int8_t attachUsCallback(void (*callbackFunction)(), uint16_t microseconds);
		static int8_t attachMsMemberCallback(ChronosCallbackOwnerClass* obj, uint16_t milliseconds);
		static int8_t attachUsMemberCallback(ChronosCallbackOwnerClass* obj, uint16_t microseconds);
		static void detachCallback(uint8_t id);
		static void delay(uint16_t delay_ms);
		static void timerHandler();

	private:

		Chronos();
	    Chronos(const Chronos&);  
	    Chronos& operator=(Chronos&);

		void processTimerHandler();
		int8_t attachLambda(void (*callbackFunction)(), uint16_t period, bool isUsScale);
		int8_t attachMemberCallback(ChronosCallbackOwnerClass* obj, uint16_t period, bool isUsScale);
		int8_t attachCallback(void (*callbackFunction)(), uint16_t period, bool isUsScale);
		int8_t pushCallbackItem(ChronosCallbackItem *item);
		int8_t deleteCallbackItem(uint8_t id);
		void walkCallback(bool isUsScale);
		uint8_t spawnNewId();

		volatile bool lock = false;
		volatile uint8_t usCounter = 0; // Actually it's not uS counter, it is tick counter where 1 tick = 100us
		volatile uint8_t usCallbackCount = 0;
		
		volatile uint16_t delayLimit = 0;
		volatile uint16_t delayCounter = 0;
		
		volatile uint8_t callbackArraySize = 0;
		ChronosCallbackItem *callbackArray;
};

#endif