#ifndef _GEEGROW_DC_MOTOR_H
#define _GEEGROW_DC_MOTOR_H
#include <Arduino.h>
#include "OpticalEncoder.h"
#include "PCA9685.h"

//#define DEBUG_MODE

enum MotorState {
	STATE_STOPPED, 
	STATE_FORWARD, 
	STATE_BACKWARD,
};

enum MotorScheduledTask {
	MOTOR_TASK_IDLE,
	MOTOR_TASK_OFF,
};

class DCMotor {
	public:
		// Instance indentifier
		uint8_t id;

		DCMotor(PCA9685* pwmCtrlr, uint8_t ch1, uint8_t ch2, uint8_t minSpeed, bool normal_direction = true, uint8_t inert_factor = 0);
		void setForwardSpeed(uint8_t speed);
		void forward(uint8_t speed);
		void forward(uint8_t speed, float rotations);
		void setBackwardSpeed(uint8_t speed);
		void backward(uint8_t speed);
		void backward(uint8_t speed, float rotations);
		void stop(bool hard = true);
		void abort();
		bool isStopped();
		bool isForward();
		bool isBackward();
		// 1 tick = 100us
		void externalTick(uint8_t &periodMs);
		void attachEncoder(OpticalEncoder* opticalEncoder);
		bool isEncoderLimitExceeded();
		bool isBusy();

	private:
		// PWM controller PCA9685
		PCA9685* pwmController;

		// Encoder on wheel
		OpticalEncoder* opticalEncoder;

		volatile MotorState state = STATE_STOPPED;
		volatile MotorScheduledTask task = MOTOR_TASK_IDLE;

		// Channels
		uint8_t channelOne;
		uint8_t channelTwo;
		bool normalDirection = true;
		uint8_t inertFactor = 0;
		uint8_t minSpeed = 0;

		// Current motor state
		uint8_t forwardSpeed = 0;
		uint8_t backwardSpeed = 0;

		// Counter of tick's. 1 tick = 100us (0.1ms). So total it can be 6553500us
		uint16_t tickCounter = 0;
		uint16_t tickLimit = 0;

		bool encoderEnabled = true;

		void off();
		uint8_t getHardStopTime();
		uint8_t normalizeSpeed(uint8_t speed);
		void checkEncoderLimit();
		uint8_t getSpeed();
		void dalayAndOff(uint8_t time_ms);
};

#endif
