#ifndef _GEEGROW_OPTICAL_MOTOR_ENCODER_H
#define _GEEGROW_OPTICAL_MOTOR_ENCODER_H
#include <Arduino.h>
#include "Chronos.h"

//#define DEBUG_MODE

#define FILTER_COUNTER_LIMIT 3
#define TICK_FACTOR 10000
#define TIMER_LIMIT 32767
#define TIMER_PERIOD 1 // msec
#define SAMPLES_COUNTER_LIMIT 25 // Because 25 * 8 = 2000; It's useful because TICK_FACTOR/2000 = 5 (not floating point value)

class OpticalEncoder : public ChronosCallbackOwnerClass {
	public:
		// Sectors per one loop of encoder wheel
		uint8_t sectorsPerLoop;

		// Length of wheel in milimeters
		uint16_t wheelLength;
		
		OpticalEncoder(uint8_t channel, uint8_t sectorsPerLoop, uint16_t wheelLength);
		
		void update();
		int32_t& getPulses();
		void getPulsesFrequency(uint16_t * freq);
		void getAngleSpeed(uint16_t * speed);
		void getSpeed(uint16_t * speed);
		void setLimit(uint16_t limit, bool isForward);
		bool isLimitExceeded();
		bool isLimited();
		void dropLimit();
		void reset();
		void enable();
		void disable();

	private:

		volatile bool enabled = false;

		volatile bool isForward = true;

		// Phisical controller port for encoder data receiving
		uint8_t channel;

		// How much of samples will be used for speed counting
		uint8_t bufferSize = 8;

		// 360deg/sectorsPerLoop
		uint8_t sectorAngle;

		// Is current sector white or black. (black = LOW, white = HIGH)
		uint8_t state = LOW;

		// Continious counter for angle calculation
		int32_t counter = 0;

		// When encoder segment changed, we don't increase counter immediately.
		// We skip about 3 miasurements
		uint8_t filterCounter = 0;

		// Positive number - counter of encoder pulses between updates of speed buffer
		uint8_t pulsesCounter = 0;

		// Positive number - counter of ticks between updates of speed buffer. 1 tick = 100us
		uint16_t timerCounter = 0;

		// Create empty buffer - max 16 elements
		uint8_t speedBuffer[8] = {0,0,0,0, 0,0,0,0};

		// Time delay between samples [ms]
		uint8_t timeBuffer[8] = {0,0,0,0, 0,0,0,0};

		// Current buffer index
		uint8_t bufferIndex = 0;

		// Number of real data in buffer
		uint8_t bufferDataLength = 0;

		// Number of samples between current and previous buffer updates
		uint8_t samplesCounter = 0;

		// Limited of wheel rotation. ABS value, independent of rotation left or right.
		uint16_t rotationLimit = 0;

		void fillBuffer();
		// static void timerHandler();
		void chronosHandler();
};

#endif
