#ifndef _GEEGROW_I2C_SERVO_H
#define _GEEGROW_I2C_SERVO_H
#include <Arduino.h>
#include "../PCA9685/PCA9685.h"
#include <Wire.h>
#include "Chronos.h"

// 1s = 1000msec
#define MS_FACTOR 1000

class I2CServo : public ChronosCallbackOwnerClass {
	public:
		I2CServo(
			PCA9685* pwmCtrlr,
			uint8_t channel,
			uint16_t min,
			uint16_t max,
			uint8_t minSpeed = 30,
			uint16_t maxSpeed = 100,
			uint8_t acceleration = 200,
			uint8_t pwmFrequency = 50
		);

		void begin(uint8_t internalTimerMs = 0);
		void writeMicroseconds(uint16_t length);
		void write(int16_t angle);
		void rotate(int16_t angle);
		void smoothRotate(int16_t srcAngle, int16_t destAngle);
		void smoothRotate(int16_t angle);
		bool isBusy();

		void externalTick(uint8_t periodMs);
		void setMinAngularSpeed(uint8_t minSpeed);
		void setMaxAngularSpeed(uint16_t maxSpeed);
		void setMaxAcceleration(uint8_t acceleration);

	private:
		// PWM controller PCA9685
		PCA9685* pwmController;

		// LSB - length in usec
		float lsb = 0;
		uint8_t internalTimerMs;
		uint8_t channel;
		uint16_t min = 0; // Degree
		uint16_t max = 0; // Degree
		uint8_t minAngularSpeed = 30; // Degree/sec
		uint16_t maxAngularSpeed = 100; // Degree/sec
		uint32_t currentAngularSpeedMsScale = 0; // Degree/msec
		uint8_t acceleration = 200; // Degree/sec^2
		uint16_t angleChangeTime = 0; // Sec

		int16_t destAngle = -1; // Degree
		int16_t currentAngle = -1; // Degree
		int16_t middleAngle = -1; // Degree
		int16_t speedDownAngle = -1; // Degree

		// Rotation direction - CW or CCW
		bool isCCW;

		void setAngle(int16_t angle);
		int16_t normalizeAngle(uint16_t angle);
		uint16_t calculateAngularSpeed(uint8_t timeMs);
		int16_t getAngle(uint8_t timeMs);
		bool isRotationOver();
		bool isMiddleAngleExceded();
		bool isSpeedDownAngleExceded();
		bool isMaxSpeedReached();
		void chronosHandler();
};

#endif
