/*************************************************** 
	This is a library for our Adafruit 16-channel PWM & Servo driver

	Pick one up today in the adafruit shop!
	------> http://www.adafruit.com/products/815

	These displays use I2C to communicate, 2 pins are required to  
	interface. For Arduino UNOs, thats SCL -> Analog 5, SDA -> Analog 4

	Adafruit invests time and resources providing this open source code, 
	please support Adafruit and open-source hardware by purchasing 
	products from Adafruit!

	Written by Limor Fried/Ladyada for Adafruit Industries.  
	BSD license, all text above must be included in any redistribution
 ****************************************************/

#ifndef _PCA9685_H
#define _PCA9685_H

#include <Arduino.h>
#include <Wire.h>

#define PCA9685_SUBADR1 0x2
#define PCA9685_SUBADR2 0x3
#define PCA9685_SUBADR3 0x4

#define PCA9685_MODE1 0x0
#define PCA9685_PRESCALE 0xFE

#define LED0_ON_L 0x6
#define LED0_ON_H 0x7
#define LED0_OFF_L 0x8
#define LED0_OFF_H 0x9

#define ALLLED_ON_L 0xFA
#define ALLLED_ON_H 0xFB
#define ALLLED_OFF_L 0xFC
#define ALLLED_OFF_H 0xFD

/**************************************************************************/
/*! 
		@brief  Class that stores state and functions for interacting with PCA9685 PWM chip
*/
/**************************************************************************/
class PCA9685 {
	public:
		PCA9685(uint8_t addr = 0x40, uint16_t frequency = 50);
		void begin(void);
		void reset(void);
		void setPWMFreq(float freq);
		void setPWM(uint8_t num, uint16_t on, uint16_t off);
		void setRelPWM(uint8_t pinNum, uint8_t PwmValue);
		void setPin(uint8_t num, uint16_t val, bool invert=false);

	private:
		// Physical I2C address of PCA9685
		uint8_t deviceAddr;
		uint16_t frequency;
		
		uint8_t read(uint8_t registerAddress);
		void write(uint8_t registerAddress, uint8_t data);
};

#endif
